#!/bin/bash

# Omwol omapi wake on lan from hostname tool.
# Copyright (C) 2009-2010  James Shubin, McGill University
# Written for McGill University by James Shubin <purpleidea@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# get the mac address corresponding to a hostname using dhcpd omshell omapi,
# and use it to run a wake on lan.
# if you specify ./$0 <prefix> <seq arg1> [<seq arg2>] then it will do a wol
# on all the hostnames in the resultant: `seq` (but with the prefix prepended)

# TODO:
# - include a generate option to generate a .sh script with the wol commands

# EXAMPLES:
# $ omwol myhost
# (wakes up mac address of host: myhost)
# $ omwol prefix- 4
# (wakes up hosts: prefix-1, prefix-2, prefix-3, prefix-4)
# $ omwol machine 2 5
# (wakes up hosts: machine2, machine3, machine4, machine5)

# usage
if [ "$1" == "" ]; then
	echo "usage: `basename $0` <hostname> [seq]"
	exit 1
fi

# omshell function
echo_hw_from_hostname() {

	# build commands for omshell
	output=''
	output=$output'server localhost\n'
	output=$output'connect\n'
	output=$output'new host\n'
	output=$output'set name = "'"$1"'"\n'
	output=$output'open\n'

	# 1) send commands to omshell
	# 2) reverse lines
	# 3) grep for hardware-address (to get the last find of it)
	# 4) keep only last line found
	# 5) break off the other pieces to be left with just the mac address
	# 6) store it in a var
	hw=`echo -e $output | omshell | tac | grep 'hardware-address' | head --lines=1 | awk '{ print $3; }'`

	# since we can't really return values in bash, echo one back, and capture it in backticks
	echo $hw
}

if [ "$2" == "" ]; then
	hw=`echo_hw_from_hostname $1`
	if [ "$hw" == "" ]; then
		echo "mac address not found for: $1"
		exit 1
	fi
	wol -h $1 $hw
else

	# send the correct num of params to seq
	if [ "$3" == "" ]; then
		seq=`seq $2`
	else
		seq=`seq $2 $3`
	fi

	for i in $seq; do
		hn=$1"$i"
		hw=`echo_hw_from_hostname $hn`
		if [ "$hw" == "" ]; then
			echo "mac address not found for: $hn"
		else
			# print out a host command
			echo -n `host $hn`', '
			# do the wake on lan
			wol -h $hn $hw
		fi
	done
fi

exit 0

